import React, { forwardRef, useImperativeHandle, useRef } from 'react';
import { findNodeHandle, NativeModules, Platform, requireNativeComponent, UIManager } from 'react-native';
const LINKING_ERROR = `The package 'react-native-camera-module' doesn't seem to be linked. Make sure: \n\n` + Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const ComponentName = 'CameraModuleView';
export const CameraModuleView = UIManager.getViewManagerConfig(ComponentName) != null ? requireNativeComponent(ComponentName) : () => {
  throw new Error(LINKING_ERROR);
};
export const CameraModule = NativeModules.CameraModuleViewManager;
const CameraView = /*#__PURE__*/forwardRef((_ref, parentRef) => {
  let {
    torch,
    device,
    style,
    zoom,
    deviceType
  } = _ref;
  const ref = useRef();

  const handle = () => {
    const nodeHandle = findNodeHandle(ref === null || ref === void 0 ? void 0 : ref.current);

    if (nodeHandle == null || nodeHandle === -1) {
      throw new Error('system/view-not-found' + "Could not get the Camera's native view tag! Does the Camera View exist in the native view-tree?");
    }

    return nodeHandle;
  };

  useImperativeHandle(parentRef, () => ({
    start: () => {
      const nodeHandle = handle();
      CameraModule.start(nodeHandle);
    },
    stop: () => {
      const nodeHandle = handle();
      CameraModule.stop(nodeHandle);
    },
    record: options => {
      const nodeHandle = handle();
      CameraModule.startRecording(nodeHandle, options, response => {
        options.recordingCallback(response);
      });
    },
    pause: () => {
      const nodeHandle = handle();
      CameraModule.pauseRecording(nodeHandle);
    },
    finish: () => {
      const nodeHandle = handle();
      CameraModule.stopRecording(nodeHandle);
    },
    getSupportedDevices: (devicePosition, callback) => {
      const nodeHandle = handle();
      return CameraModule.getDevices(nodeHandle, devicePosition, callback);
    }
  }));
  return /*#__PURE__*/React.createElement(CameraModuleView, {
    ref: ref,
    device: device,
    torch: torch,
    style: style ? style : {},
    zoom: zoom,
    deviceType: deviceType
  });
});
export default CameraView;
//# sourceMappingURL=CameraView.js.map